/*
 *
 * Copyright  2005, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

#include "MouseCtrl.h"

namespace ColorSelectorCpp
{

	// constructor
	MouseCtrl::MouseCtrl(Form * mainForm)
	{
		p_mForm = mainForm;
	}


	System::Void MouseCtrl::MoveMouse(int dx, int dy)
	{
		// get the cursor's current position
		Point current = Cursor::Position;

		// update the cursor's postion
		Cursor::Position = Point( current.X + dx,current.Y + dy );
	}

	System::Void MouseCtrl::MouseClick()
	{

		// Determine where the mouse is currently located
		Point clickPoint = Cursor::Position;

		// Determine what control is located under the cursor
		Control * c = p_mForm->GetChildAtPoint(Point(clickPoint.X - p_mForm->Left - 3, clickPoint.Y - p_mForm->Top - 20)); // YUP magic constants
		// c might be undefined if there is no ctrl under mouse, so try
		try {
			// Locate PerformClick method on c
			System::Reflection::MethodInfo * performClick = c->GetType()->GetMethod(S"PerformClick");
			try {
				System::Object * params[];
				// try calling PerformClick method, will throw nullReferenceException
				// if PerformClick method doesn't exist for c
				performClick->Invoke(dynamic_cast<System::Object*>(c), params );
			} 
			catch (System::NullReferenceException * e) 
			{
				// debug prints
				//MessageBox::Show(String::Concat("Performclick not found for control; ", c->ToString()));
			}

		} 
		catch (System::NullReferenceException * e) // no control at current coords
		{
			// debug prints
			//MessageBox::Show(String::Concat("No Control found at x= ", clickPoint.X.ToString(), " y= ", clickPoint.Y.ToString()));
		}
	}
}